# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.pricing.v2.voice.country import CountryList
from twilio.rest.pricing.v2.voice.number import NumberList


class VoiceList(ListResource):

    def __init__(self, version):
        """
        Initialize the VoiceList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.pricing.v2.voice.VoiceList
        :rtype: twilio.rest.pricing.v2.voice.VoiceList
        """
        super(VoiceList, self).__init__(version)

        # Path Solution
        self._solution = {}

        # Components
        self._countries = None
        self._numbers = None

    @property
    def countries(self):
        """
        Access the countries

        :returns: twilio.rest.pricing.v2.voice.country.CountryList
        :rtype: twilio.rest.pricing.v2.voice.country.CountryList
        """
        if self._countries is None:
            self._countries = CountryList(self._version, )
        return self._countries

    @property
    def numbers(self):
        """
        Access the numbers

        :returns: twilio.rest.pricing.v2.voice.number.NumberList
        :rtype: twilio.rest.pricing.v2.voice.number.NumberList
        """
        if self._numbers is None:
            self._numbers = NumberList(self._version, )
        return self._numbers

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Pricing.V2.VoiceList>'


class VoicePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the VoicePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.pricing.v2.voice.VoicePage
        :rtype: twilio.rest.pricing.v2.voice.VoicePage
        """
        super(VoicePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of VoiceInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.pricing.v2.voice.VoiceInstance
        :rtype: twilio.rest.pricing.v2.voice.VoiceInstance
        """
        return VoiceInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Pricing.V2.VoicePage>'


class VoiceInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the VoiceInstance

        :returns: twilio.rest.pricing.v2.voice.VoiceInstance
        :rtype: twilio.rest.pricing.v2.voice.VoiceInstance
        """
        super(VoiceInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'name': payload.get('name'),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def name(self):
        """
        :returns: The resource name
        :rtype: unicode
        """
        return self._properties['name']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: The URLs of the related Countries and Numbers resources
        :rtype: unicode
        """
        return self._properties['links']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Pricing.V2.VoiceInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class UserBindingList(ListResource):

    def __init__(self, version, service_sid, user_sid):
        """
        Initialize the UserBindingList

        :param Version version: Version that contains the resource
        :param service_sid: The service_sid
        :param user_sid: The user_sid

        :returns: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingList
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingList
        """
        super(UserBindingList, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'user_sid': user_sid, }
        self._uri = '/Services/{service_sid}/Users/{user_sid}/Bindings'.format(**self._solution)

    def stream(self, binding_type=values.unset, limit=None, page_size=None):
        """
        Streams UserBindingInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param list[UserBindingInstance.BindingType] binding_type: The binding_type
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(binding_type=binding_type, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, binding_type=values.unset, limit=None, page_size=None):
        """
        Lists UserBindingInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param list[UserBindingInstance.BindingType] binding_type: The binding_type
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance]
        """
        return list(self.stream(binding_type=binding_type, limit=limit, page_size=page_size, ))

    def page(self, binding_type=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of UserBindingInstance records from the API.
        Request is executed immediately

        :param list[UserBindingInstance.BindingType] binding_type: The binding_type
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of UserBindingInstance
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingPage
        """
        data = values.of({
            'BindingType': serialize.map(binding_type, lambda e: e),
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return UserBindingPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of UserBindingInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of UserBindingInstance
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return UserBindingPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a UserBindingContext

        :param sid: The sid

        :returns: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingContext
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingContext
        """
        return UserBindingContext(
            self._version,
            service_sid=self._solution['service_sid'],
            user_sid=self._solution['user_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a UserBindingContext

        :param sid: The sid

        :returns: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingContext
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingContext
        """
        return UserBindingContext(
            self._version,
            service_sid=self._solution['service_sid'],
            user_sid=self._solution['user_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.IpMessaging.V2.UserBindingList>'


class UserBindingPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the UserBindingPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param service_sid: The service_sid
        :param user_sid: The user_sid

        :returns: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingPage
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingPage
        """
        super(UserBindingPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of UserBindingInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance
        """
        return UserBindingInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            user_sid=self._solution['user_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.IpMessaging.V2.UserBindingPage>'


class UserBindingContext(InstanceContext):

    def __init__(self, version, service_sid, user_sid, sid):
        """
        Initialize the UserBindingContext

        :param Version version: Version that contains the resource
        :param service_sid: The service_sid
        :param user_sid: The user_sid
        :param sid: The sid

        :returns: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingContext
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingContext
        """
        super(UserBindingContext, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'user_sid': user_sid, 'sid': sid, }
        self._uri = '/Services/{service_sid}/Users/{user_sid}/Bindings/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the UserBindingInstance

        :returns: The fetched UserBindingInstance
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return UserBindingInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            user_sid=self._solution['user_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the UserBindingInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.IpMessaging.V2.UserBindingContext {}>'.format(context)


class UserBindingInstance(InstanceResource):

    class BindingType(object):
        GCM = "gcm"
        APN = "apn"
        FCM = "fcm"

    def __init__(self, version, payload, service_sid, user_sid, sid=None):
        """
        Initialize the UserBindingInstance

        :returns: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance
        """
        super(UserBindingInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'endpoint': payload.get('endpoint'),
            'identity': payload.get('identity'),
            'user_sid': payload.get('user_sid'),
            'credential_sid': payload.get('credential_sid'),
            'binding_type': payload.get('binding_type'),
            'message_types': payload.get('message_types'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {
            'service_sid': service_sid,
            'user_sid': user_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: UserBindingContext for this UserBindingInstance
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingContext
        """
        if self._context is None:
            self._context = UserBindingContext(
                self._version,
                service_sid=self._solution['service_sid'],
                user_sid=self._solution['user_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The sid
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The account_sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: The service_sid
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def date_created(self):
        """
        :returns: The date_created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date_updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def endpoint(self):
        """
        :returns: The endpoint
        :rtype: unicode
        """
        return self._properties['endpoint']

    @property
    def identity(self):
        """
        :returns: The identity
        :rtype: unicode
        """
        return self._properties['identity']

    @property
    def user_sid(self):
        """
        :returns: The user_sid
        :rtype: unicode
        """
        return self._properties['user_sid']

    @property
    def credential_sid(self):
        """
        :returns: The credential_sid
        :rtype: unicode
        """
        return self._properties['credential_sid']

    @property
    def binding_type(self):
        """
        :returns: The binding_type
        :rtype: UserBindingInstance.BindingType
        """
        return self._properties['binding_type']

    @property
    def message_types(self):
        """
        :returns: The message_types
        :rtype: list[unicode]
        """
        return self._properties['message_types']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the UserBindingInstance

        :returns: The fetched UserBindingInstance
        :rtype: twilio.rest.ip_messaging.v2.service.user.user_binding.UserBindingInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the UserBindingInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.IpMessaging.V2.UserBindingInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.pricing.v2.country import CountryList
from twilio.rest.pricing.v2.number import NumberList
from twilio.rest.pricing.v2.voice import VoiceList


class V2(Version):

    def __init__(self, domain):
        """
        Initialize the V2 version of Pricing

        :returns: V2 version of Pricing
        :rtype: twilio.rest.pricing.v2.V2.V2
        """
        super(V2, self).__init__(domain)
        self.version = 'v2'
        self._countries = None
        self._numbers = None
        self._voice = None

    @property
    def countries(self):
        """
        :rtype: twilio.rest.pricing.v2.country.CountryList
        """
        if self._countries is None:
            self._countries = CountryList(self)
        return self._countries

    @property
    def numbers(self):
        """
        :rtype: twilio.rest.pricing.v2.number.NumberList
        """
        if self._numbers is None:
            self._numbers = NumberList(self)
        return self._numbers

    @property
    def voice(self):
        """
        :rtype: twilio.rest.pricing.v2.voice.VoiceList
        """
        if self._voice is None:
            self._voice = VoiceList(self)
        return self._voice

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Pricing.V2>'
